"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResultUtils = void 0;
const delay_1 = __importDefault(require("delay"));
const neverthrow_1 = require("neverthrow");
class ResultUtils {
    static combine(asyncResultList) {
        return neverthrow_1.ResultAsync.fromPromise(Promise.all(asyncResultList), (e) => {
            return e;
        }).andThen(ResultUtils.combineResultList);
    }
    static combineResultList(resultList) {
        return resultList.reduce((acc, result) => {
            return acc.isOk()
                ? result.isErr()
                    ? neverthrow_1.err(result.error)
                    : acc.map((values) => {
                        values.push(result.value);
                        return values;
                    })
                : acc;
        }, neverthrow_1.ok([]));
    }
    static fromThrowableResult(throwableCallback) {
        const throwable = neverthrow_1.Result.fromThrowable(throwableCallback, (err) => {
            return err;
        });
        return throwable();
    }
    static executeSerially(funcList) {
        // const results = new Array<T>();
        // // for (const func of funcList) {
        // //   func().map((val) => {})
        // // }
        // try {
        //   funcList.reduce(
        //     (p: ResultAsync<void, E>, x) =>
        //       p.andThen(_ => {
        //         return x()
        //           .map((result) => {
        //             results.push(result);
        //           })
        //           .mapErr((e) => {
        //             throw e;
        //           });
        //       }),
        //     okAsync<void, E>(undefined)
        //   );
        // }
        // catch (e) {
        //   return errAsync(e);
        // }
        const func = () => __awaiter(this, void 0, void 0, function* () {
            const results = new Array();
            for (const func of funcList) {
                const result = yield func();
                if (result.isErr()) {
                    throw result.error;
                }
                else {
                    results.push(result.value);
                }
            }
            return results;
        });
        return neverthrow_1.ResultAsync.fromPromise(func(), (e) => {
            return e;
        });
    }
    static backoffAndRetry(func, acceptableErrors, maxAttempts, baseSeconds = 5) {
        if (maxAttempts != null && maxAttempts < 1) {
            throw new Error("maxAttempts must be 1 or more!");
        }
        if (baseSeconds < 1) {
            throw new Error("baseSeconds must be 1 or more!");
        }
        const runAndCheck = (currentAttempt, nextAttemptSecs, lastError) => {
            if (maxAttempts != null && currentAttempt > maxAttempts) {
                if (lastError == null) {
                    throw new Error("Error before first function run; logical error! maxAttempts must be 1 or more!");
                }
                return neverthrow_1.errAsync(lastError);
            }
            // Check the result. If it's not an error, we're done!
            // If it's an error, check the error type against acceptableErrors. If it's in the list,
            // wait some amount of time and try again.
            // If it is not in the list, return the error and stop.
            return func().orElse((e) => {
                let retry = false;
                for (const acceptableError of acceptableErrors) {
                    if (e instanceof acceptableError) {
                        retry = true;
                        break;
                    }
                }
                if (retry) {
                    return neverthrow_1.ResultAsync.fromSafePromise(delay_1.default(nextAttemptSecs)).andThen(() => {
                        return runAndCheck(++currentAttempt, nextAttemptSecs * 2, e);
                    });
                }
                return neverthrow_1.errAsync(e);
            });
        };
        return runAndCheck(1, baseSeconds, null);
    }
}
exports.ResultUtils = ResultUtils;
//# sourceMappingURL=ResultUtils.js.map